@@ -12,7 +12,7 @@ Created on 27 Sep 2020
 @author: semuadmin
 '''
 
-from pyubx2.ubxtypes_core import U1, I1, X1, U2, I2, X2, U4, I4 , X4 , R4 , X6, R8 , C06, C10, C30, C32, CH
+from pyubx2.ubxtypes_core import U1, I1, X1, U2, I2, X2, U3, U4, I4, U5, X4, R4, U6, X6, R8, C06, C10, C30, C32, CH
 
 UBX_PAYLOADS_POLL = {
 'AID-ALM': {
